set more 1


import excel using data.xlsx, firstrow clear sheet("Quarterly")
destring _all, force replace
sort yy



gen time=_n
tsset time
*Parameters
local gamma=3
local minall=0
local maxall=1.5

*Set periods
local samplestart=19471
local firstpred=19651
local lastpred=20054

*Set periods via time index variable
sum time if yy==`samplestart'
local tsamplestart=r(mean)
sum time if yy==`firstpred'
local tfirstpred=r(mean)
sum time if yy==`lastpred'
local tlastpred=r(mean)


*Define variables
gen dp=ln(D12/Index)
gen dy=ln(D12/Index[_n-1])
gen ep=ln(E12/Index)
gen dpay=ln(D12/E12)
gen stockvar=svar
gen booktomarket=bm
gen netequityexp=ntis
gen treasury=tbl
gen longtermyield=lty
gen longtermreturn=ltr
gen termspread=lty-tbl
gen defyieldspread=BAA-AAA
gen defratespread=corpr-ltr
gen inflation=infl[_n-1]
gen ikt=ik


gen ret=(Index+D12/4)/Index[_n-1]-1
replace ret=CRSP_SPvw if CRSP_SPvw!=.

gen exret=ret-Rfree[_n-1]
gen riskfree=tbl[_n-1]/4

local theta=0.9
gen theta=`theta'^40 if yy==19551
replace theta=theta[_n-1]/`theta' if yy>19551

gen phitotal=0
gen sphitotal=0


*Produce individual out-of-sample forecasts
foreach var of varlist dp-ikt{
	di `var'
	qui{
	gen sfxx`var'=.
	gen dfxx`var'=.
	gen sthetafxx`var'=0
	gen sphifxx`var'=0
	gen sdfxx`var'=.
	
	
	forvalues i=`tsamplestart'/`tlastpred'{
		*with simple returns
		capture drop x
		capture reg exret l.`var' if yy>`samplestart' & time<`i'
		capture predict x
		capture replace sfxx`var'=x if time==`i'
		
		capture replace sthetafxx`var'=sthetafxx`var'[_n-1]+(x-exret)^2*theta if time==`i' & theta!=.
		capture replace sphifxx`var'=1/(sthetafxx`var'/theta) if time==`i' & theta!=.
		capture replace sphitotal=sphitotal+sphifxx`var' if time==`i'
		capture replace sdfxx`var'=x*sphifxx`var'[_n-1] if time==`i'
		
	
		}
		
		
		
	}
}

*Generate mean forecasts, variance, and trailing means
*egen pred=rowmean(fxx*)
egen pred=rowtotal(dfxx*)
replace pred=pred/phitotal[_n-1]


egen spred=rowtotal(sdfxx*)
replace spred=spred/sphitotal[_n-1]



gen smodelvar=.
gen smodelmean=.



forvalues i=`tsamplestart'/`tlastpred'{
		
		
		sum exret if yy>=`samplestart' & time<`i'
		replace smodelmean=r(mean) if time==`i'

		
		*Dividing by N instead of N-1 follows the coding for a paper co-authored by Rapach and Zhou
		sum exret if time>=`i'-40 & time<`i'
		replace smodelvar=r(Var)*(r(N)-1)/r(N) if time==`i'
	
		

}




*Performance analysis

matrix df = (1, -1, -`gamma'/2, `gamma'/2)
local z=1


	*Determine optimal weights and strategy returns
	foreach var of varlist spred smodelmean{
		gen wopt`var'= max(`minall',min(`maxall',`var'/`gamma'/smodelvar))
		gen opt`var'= wopt`var'*exret+riskfree
	}
	gen woptbuyhold=1
	gen optbuyhold=woptbuyhold*exret+riskfree

	
	
*Determine Utility Gains via certainty equivalents cee ) and Sharpe ratios


local vars `" "optspred" "optsmodelmean"  "optbuyhold" "'
 
foreach var of local vars{
	
	*Utility over entire period
	sum `var' if yy>=`firstpred' & time<=`tlastpred'
	local cee`var'=r(mean)-0.5*`gamma'*r(Var)
	local m`var'=r(Var)
	local n=r(N)
	
	

	*realized utility	
	gen ru`var'=`var'-0.5*`gamma'*w`var'^2*svar
	
		
	*Shrape Ratios
	capture gen sharpe`var'=.
	gen e`var'=`var'-riskfree
	sum e`var' if yy>=`firstpred' & time<=`tlastpred'
	replace sharpe`var'=r(mean)/r(sd)*(4)^0.5 in `z'

	
	local z1=`z'+1
	*Test for Sharpe Ratio
	srtest eoptspred e`var' if yy>=`firstpred' & time<=`tlastpred'
	replace sharpe`var'=r(p) in `z1'
	
	
	*Test Delta Method
	cor optspred `var' if yy>=`firstpred' & yy<=`lastpred' & time<=`tlastpred', cov
	local c=_result(4)
	
	capture gen du`var'=.
	replace du`var'=(`ceeoptspred'-`cee`var'')*400 in `z'
	
	
	matrix  omega = (`moptspred',`c',0,0 \ `c',`m`var'',0,0 \ 0,0,2*`moptspred'^2,2*`c'^2 \ 0,0,2*`c'^2,2*`m`var''^2)
		
	matrix p=df*omega*df'	
	replace du`var'=2*normprob(-abs(du`var'[`z']/400/(p[1,1])^.5*`n'^0.5)) in `z1'
	
	local z2=`z'+2
	*T-test with realized utility
	ttest ruoptspred=ru`var' if yy>=`firstpred' & yy<=`lastpred' & time<=`tlastpred'
	replace du`var'=r(p) in `z2'
	
	
	
	
	
	
}



gen what="RSZ dmspe forecast" in 1
export excel what du* sharpe* using Paperresults.xlsx in 1/30, sheetmod sheet("Table1") cell(A6) first(var)
