set more 1

import excel using data.xlsx, firstrow clear sheet("Monthly")
destring _all, force replace
sort yy



gen time=_n
tsset time

*Sample length and length of initial estimation period (in mnonths)
local total=600
local estperiod=240


*Length of window for rolling variance estimation (in mnonths)
local varest=60


*Portfolio Restrictions
local minall=0
local maxall=1.5


*Constant Variance
local varcon=0.15^2/12

*Set periods
local samplestart=192601
local end1=202012

sum time if yy==`samplestart'
local tsamplestart=r(mean)
sum time if yy==`end1'
local tend1=r(mean)





*gen ret=(Index+D12/12)/Index[_n-1]-1
gen ret=CRSP_SPvw 

gen exret=ret-Rfree[_n-1]
gen riskfree=Rfree[_n-1]


*Initialize variables
forvalues gamma=2/5{
	gen du`gamma'=.
	gen dsharpe`gamma'=.
	gen varweight`gamma'=.
	gen meanweight`gamma'=.
}
forvalues gamma=2/5{
	gen duc`gamma'=.
	gen dsharpec`gamma'=.
}
forvalues gamma=2/5{
	gen varweightc`gamma'=.
	gen meanweightc`gamma'=.
}


local end=`tend1'-`total'

gen modelmean=.
gen modelvar=.
gen buyhold=ret
gen ebuyhold=ret-riskfree


	
forvalues s=`tsamplestart'/`end'{

di `s'	
		capture drop x*
		
		qui{
		local istart=`s' + `estperiod'
		local iend=`s' + `total' - 1
		forvalues i=`istart'/`iend'{
			
			
			sum exret if time>=`s' & time<`i'
			replace modelmean=r(mean) if time==`i'
			sum exret if time>=`i'-`varest' & time<`i'
			replace modelvar=r(Var) if time==`i'
			
			
		}	

		
			
		
		forvalues gamma=2/5{
			capture drop x*
			sum buyhold if time>=`istart' & time<=`iend'
			local ceebuy=r(mean)-0.5*`gamma'*r(Var)
			sum ebuyhold if time>=`istart' & time<=`iend'
			local sharpebuy=r(mean)/r(sd)*12^0.5
		
			
			
			gen xwoptmean= max(`minall',min(`maxall',modelmean/`gamma'/modelvar))
			gen xoptmean=xwoptmean*exret+riskfree
			gen xeoptmean=xoptmean-riskfree
			
			sum xoptmean if time>=`istart' & time<=`iend'
			local ceemean=r(mean)-0.5*`gamma'*r(Var)
						
			replace du`gamma'=(`ceebuy'-`ceemean')*12 if time==`s'
			
			sum xwoptmean if time>=`istart' & time<=`iend'
			replace varweight`gamma'=r(sd) if time==`s'
			replace meanweight`gamma'=r(mean) if time==`s'
			
			sum xeoptmean if time>=`istart' & time<=`iend'
			local sharpemean=r(mean)/r(sd)*12^0.5
						
			replace dsharpe`gamma'=`sharpebuy'-`sharpemean' if time==`s'
			
			*Same for constant variance assumption
			capture drop x*
			gen xwoptmean= max(`minall',min(`maxall',modelmean/`gamma'/`varcon'))
			gen xoptmean=xwoptmean*exret+riskfree
			gen xeoptmean=xoptmean-riskfree
			
			sum xoptmean if time>=`istart' & time<=`iend'
			local ceemean=r(mean)-0.5*`gamma'*r(Var)
			
			sum xwoptmean if time>=`istart' & time<=`iend'
			replace varweightc`gamma'=r(sd) if time==`s'
			replace meanweightc`gamma'=r(mean) if time==`s'
						
			replace duc`gamma'=(`ceebuy'-`ceemean')*12 if time==`s'
			
			sum xeoptmean if time>=`istart' & time<=`iend'
			local sharpemean=r(mean)/r(sd)*12^0.5
						
			replace dsharpec`gamma'=`sharpebuy'-`sharpemean' if time==`s'
			
		}
		
		
		
		
		}
	
		
		
		
		
}

local y=`estperiod'/12
local x=(`total'-`estperiod')/12
local s=`"Figure`y'`x'"'

export excel yy du* dsharpe* varweight* meanweight* using Paperresults.xlsx if du3!=., sheetmod sheet(`"`s'"') cell(B1) first(var)
